#include <types.h>
#include <lib.h>
#include <Interrupts/interrupts.h>

#include "atapio.h"

uint32_t ata_read_sectors(uint8_t drive, uint32_t lba, uint8_t count, uint16_t* buf)
{
	if (!buf)
		return -1;

	if (drive == ATA_MASTER) {
		outb(0xE0 | ((lba >> 24) & 0x0f), ATA_DRIVE_SEL);
	} else if (drive == ATA_SLAVE) {

	} else {
		printf("Invalid drive.\n");
		return -1;
	}

	outb(count, ATA_SECTOR_CNT);
	outb((uint8_t)lba, ATA_LBA_LOW);
	outb((uint8_t)(lba >> 8), ATA_LBA_MID);
	outb((uint8_t)(lba >> 16), ATA_LBA_HIGH);
	outb(0x20, ATA_COMMAND);

	int j;
	for (j = 0; j < count; j++){
		uint8_t status;
		status = inb(ATA_STATUS);
		while ((status & ATA_STATUS_BSY) || !(status & ATA_STATUS_DRQ)) {
			status = inb(ATA_STATUS);
		}

		int i;
		for (i = 0; i < 256; i++) {
			buf[i+j*256] = inw(ATA_DATA);
		}
	}

	return 0;	
}

uint32_t atapio_identify(uint8_t drive, uint16_t* buf)
{

	if (drive == 0) {
		outb(0xA0, ATA_DRIVE_SEL);
	} else if (drive == 1) {
		outb(0xB0, ATA_DRIVE_SEL);
	} else {
		printf("Invalid drive\n");
		return -1;
	}

	outb(0, ATA_SECTOR_CNT);
	outb(0, ATA_LBA_LOW);
	outb(0, ATA_LBA_MID);
	outb(0, ATA_LBA_HIGH);

	outb(0xEC, ATA_COMMAND);

	uint8_t status = inb(ATA_STATUS);

	if (status)
		printf("Drive %d detected, status=%x\n", drive, status);

	printf("Wating for drive to not be busy... ");

	while ((status = inb(ATA_STATUS)) & 0x80);

	printf("done.\n");

	printf("Checking LBA ports... ");
	if (inb(ATA_LBA_MID) || inb(ATA_LBA_HIGH)) {
		printf("Non ATA compliant drive.\n");
		return -1;
	} else {
		printf("done.\n");
	}


	status = inb(ATA_STATUS);
	while (!(status & 8) && !(status & 1));

	if (status & 1) {
		printf("Drive error.\n");
		return -1;
	}

	printf("ATA Drive %d is ready.\n");

	if (!buf)
		return 1;

	int i;
	for (i = 0; i < 256; i++) {
		buf[i] = inw(ATA_DATA);
	}

	return 0;
}

uint32_t ata_info(uint8_t drive) {
	uint16_t ata_data[256];

	uint32_t result = atapio_identify(drive, ata_data);

	if (result)
		return result;


	printf("Drive model number: ");
	int i;
	for (i = 0; i < 20; i++) {
		printf("%c", ata_data[i+27] >> 8);
		printf("%c", ata_data[i+27] & 0xff);
	}

	printf("\n");

	printf("Drive firmware revision: ");
	for (i = 0; i < 4; i++) {
		printf("%c", ata_data[i+23] >> 8);
		printf("%c", ata_data[i+23] & 0xff);
	}

	printf("\n");

	printf("Drive serial number: ");
	for (i = 0; i < 10; i++) {
		printf("%c", ata_data[i+10] >> 8);
		printf("%c", ata_data[i+10] & 0xff);
	}

	printf("\n");

	printf("Total sector count: %d\n", ata_data[60] + (ata_data[61] << 16));
	printf("Disk size: %d\n", (ata_data[60] + (ata_data[61] << 16)) * 512);

	return 0;
}