#include <types.h>
#include <Interrupts/interrupts.h>
#include <Interrupts/i8259.h>
#include <lib.h>

#include "keyboard.h"

#define DEBUG 0

static irq_callback_t kbd_callback;

static volatile int flag = 0;
static uint8_t last_key;

static uint8_t lshift_flag = 0;
static uint8_t rshift_flag = 0;


// The normal letters numbers and punctuation symbols in scan set 1
static const char normal_map[] = {
    0, 0, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=', 0x08,
    0x09, 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', 0x0a,
    0, 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', '\'', '`', 0,
    '\\', 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', 0,
    0, 0, ' ', 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0,
    '7', '8', '9', '-',
    '4', '5', '6', '+',
    '1', '2', '3',
    '0', '.',
    0, 0, 0, 0, 0, 0, 0, 0,
};

/* scan set 1 with shift pressed */
static const char shift_map[] = {
    0, 0, '!', '@', '#', '$', '%', '^', '&', '*', '(', ')',  '_', '+', 0x08,
    0x09, 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '{', '}', 0x0a,
    0, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':', '\"', '~', 0,
    '|', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '?', 0,
    0, 0, ' ', 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0,
    '7', '8', '9', '-',
    '4', '5', '6', '+',
    '1', '2', '3',
    '0', '.',
    0, 0, 0, 0, 0, 0, 0, 0,
};


uint32_t keyboard_init()
{
	kbd_callback.handler = keyboard_handle_irq;
	kbd_callback.next = NULL;

	i8259_enable_irq(KEYBOARD_IRQ);
	attach_interrupt(KEYBOARD_IRQ, 0, &kbd_callback);
	return 0;
}

uint32_t keyboard_handle_irq()
{
	if (DEBUG)
		printf("inside keyboard.c\n");

	uint8_t scancode = inb(0x60);

	if (!(scancode & 0x80)) {
		if (scancode == LSHIFT) {
			lshift_flag = 1;
		} else if (scancode == RSHIFT) {
			rshift_flag = 1;
		} else if (scancode == LALT) {

		} else {
			if (lshift_flag || rshift_flag) {
				last_key = shift_map[scancode];
			} else {
				last_key = normal_map[scancode];
			}
			flag = 1;
		}
	} else {
		scancode &= ~0x80;
		if (scancode == LSHIFT) {
			lshift_flag = 0;
		} else if (scancode == RSHIFT) {
			rshift_flag = 0;
		}
	}

	i8259_send_eoi(KEYBOARD_IRQ);
	return 0;
}

uint8_t keyboard_get_key()
{
	while (!flag) {
		asm volatile ("hlt");
	}
	flag = 0;

	return last_key;
}

uint8_t keyboard_last_key()
{
	return last_key;
}