#ifndef _EXT2_H
#define _EXT2_H

#include <types.h>

typedef struct superblock {
	union {
		uint16_t data16[512];
		uint8_t data8[1024];
		struct {
			uint32_t inode_count;
			uint32_t block_count;
			uint32_t superuser_blocks;
			uint32_t unallocated_blocks;
			uint32_t unallocated_inodes;
			uint32_t superblock_block_num;
			uint32_t block_size_shift;
			uint32_t fragment_size_shift;
			uint32_t blocks_per_group;
			uint32_t fragments_in_group;
			uint32_t inodes_per_group;
			uint32_t last_mount_time;
			uint32_t last_written_time;
			uint16_t mounts_since_check;
			uint16_t mounts_before_check;
			uint16_t signature;
			uint16_t state;
			uint16_t error_handler;
			uint16_t minor_version;
			uint32_t last_check_time;
			uint32_t check_interval;
			uint32_t os_id;
			uint32_t major_version;
			uint16_t reserved_uid;
			uint16_t reserved_gid;
			uint32_t first_non_reserved_inode;
			uint16_t inode_size;
			uint16_t group_of_this_supernode;
			uint32_t optional_features;
			uint32_t required_features;
			uint32_t ro_features;
			uint8_t system_id[16];
			uint8_t volume_name[16];
			uint8_t last_mount_path[64];
		} __attribute__ ((packed));
	};
} superblock_t;

typedef struct block_group_descriptor {
	union {
		uint8_t data8[32];
		uint16_t data16[16];
		struct {
			uint32_t block_usage_bitmap_block_addr;
			uint32_t inode_usage_bitmap_block_addr;
			uint32_t inode_table_block_addr;
			uint16_t unallocated_blocks;
			uint16_t unallocated_inodes;
			uint16_t directories;
		} __attribute__ ((packed));
	};
} block_group_descriptor_t;

#define EXT2_FIFO 0x1000
#define EXT2_CHARDEV 0x2000
#define EXT2_DIR 0x4000
#define EXT2_BLKDEV 0x6000
#define EXT2_FILE 0x8000
#define EXT2_SYMLINK 0xA000
#define EXT2_SOCKET 0xC000

#define EXT2_OTHER_EXEC 0x001
#define EXT2_OTHER_WRITE 0x002
#define EXT2_OTHER_READ 0x004
#define EXT2_GROUP_EXEC 0x008
#define EXT2_GROUP_WRITE 0x010
#define EXT2_GROUP_READ 0x020
#define EXT2_USER_EXEC 0x040
#define EXT2_USER_WRITE 0x080
#define EXT2_USER_READ 0x100
#define EXT2_STICKY 0x200
#define EXT2_SET_GID 0x400
#define EXT2_SET_UID 0x800

typedef struct inode {
	union {
		uint8_t data8[128];
		uint16_t data16[64];
		struct {
			uint16_t type_permissions;
			uint16_t uid;
			uint32_t size_low;
			uint32_t last_access_time;
			uint32_t creation_time;
			uint32_t modification_time;
			uint32_t deletion_time;
			uint16_t gid;
			uint16_t hard_link_count;
			uint32_t sectors_used;
			uint32_t flags;
			uint32_t os_value_1;
			uint32_t block_pointer[12];
			uint32_t singly_indirect_pointer;
			uint32_t doubly_indirect_pointer;
			uint32_t triply_indirect_pointer;
			uint32_t generation;
			uint32_t reserved_ACL;
			uint32_t reserved_size;
			uint32_t fragment_block_addr;
			uint32_t os_value_2[3];
		}  __attribute__ ((packed));
	};
} inode_t;

typedef struct dir_entry {
	uint32_t inode;
	uint16_t size;
	uint8_t length;
	uint8_t type;
	uint8_t name[];
} dir_entry_t;

uint32_t ext2_init(uint32_t base);
uint32_t ext2_get_inode(uint32_t inode_num, inode_t* buf);
uint32_t ext2_read_sectors(uint32_t inode_num, uint32_t start, uint32_t count, uint16_t* buf);
uint32_t ext2_read_data(uint32_t inode_num, uint32_t count, uint16_t* buf);
uint32_t ext2_find_inode_num(const int8_t* filename);
uint32_t ext2_find_inode(const int8_t* filename, inode_t* inode);
uint32_t ext2_parse_path_to_inode_num(int8_t* filename);

#endif