#include <types.h>
#include <lib.h>

#include "i8259.h"

uint32_t i8259_init(void)
{
	cli();

	printf("Starting PIC init...\n");

	outb(MASK_ALL, MASTER_DATA);
	outb(MASK_ALL, SLAVE_DATA);

	printf("PIC config byte: %x\n", inb(MASTER_DATA));

	outb(ICW1, MASTER_CMD);           //Init master
	outb(ICW2_MASTER, MASTER_DATA);         //IRQ 7 mapped from 0x20-0x27
	outb(ICW3_MASTER, MASTER_DATA);         //Tell master that slave is on port 4
	outb(ICW4, MASTER_DATA); 


	outb(ICW1, SLAVE_CMD);            //Init Slave
	outb(ICW2_SLAVE, SLAVE_DATA);           //IRQ-7 mapped 0x28 to 0x2f
	outb(ICW3_SLAVE, SLAVE_DATA);           //Slave is master on IR2
	outb(ICW4, SLAVE_DATA);

	outb(MASK_ALL, MASTER_DATA);            //masks interrupts on PIC
	outb(MASK_ALL, SLAVE_DATA); 

	i8259_enable_irq(IRQ2_SLAVE_PIN);             // enable slave PIC

	sti();
	printf("Ending PIC INIT\n");

	return 0;

}

uint32_t i8259_enable_irq(uint32_t num)
{
	uint16_t port;
	uint8_t value;

	/* boundary check */
	if (num > 15 || num < 0){
		printf("enable_irq: invalid irq %n\n", num);
		return 1;
	}

	/* select master or slave chip and pin on chip */
	if(num < 8) {
		port = MASTER_DATA;
	} else {
		port = SLAVE_DATA;
		num -= 8;
	}

	/* select and write unmasked version */
	value = inb(port) & ~(1 << num); 
	outb(value, port);

	return 0;

}

uint32_t i8259_disable_irq(uint32_t num)
{
	uint16_t port;
	uint8_t value;

	/* boundary check */
	if (num > 15 || num < 0){
		printf("enable_irq: invalid irq %n\n", num);
		return 1;
	}

	/* select master or slave chip and pin on chip */
	if(num < 8) {
		port = MASTER_DATA;
	} else {
		port = SLAVE_DATA;
		num -= 8;
	}

	/* select and write unmasked version */
	value = inb(port) | (1 << num); 
	outb(value, port);

	return 0;
}

uint32_t i8259_send_eoi(uint32_t num)
	{
	// Code from: https://wiki.osdev.org/PIC  

	/* check for invalid */
	if (num > 15 || num < 0){
		printf("send_eoi: invalid irq %n\n", num);
		return 1;
	}

	if(num >= 8){
		/* slave PIC */
		outb((num-8)|EOI,SLAVE_CMD);
		outb(IRQ2_SLAVE_PIN|EOI, MASTER_CMD); //Slave connected to pin 2
	}else{
		/* master PIC */
		outb(num|EOI,MASTER_CMD);
	}

	return 0;
}
