/* interrupt controller chips */

#ifndef _I8259_H
#define _I8259_H

#include <types.h>

/* Ports that each PIC sits on */
#define MASTER_CMD    0x20
#define SLAVE_CMD     0xA0

#define MASTER_DATA	(MASTER_CMD+1)     // data port for master
#define SLAVE_DATA	(SLAVE_CMD+1)      // data port for slave


/* Initialization control words to init each PIC.
 * See the Intel manuals for details on the meaning
 * of each word */
#define ICW1                0x11
#define ICW2_MASTER         0x20
#define ICW2_SLAVE          0x28
#define ICW3_MASTER         0x04
#define ICW3_SLAVE          0x02
#define ICW4                0x01

#define ICW4_MASTER         0x03
#define ICW4_SLAVE          0x01

#define IRQ2_SLAVE_PIN      0x02

/* End-of-interrupt byte.  This gets OR'd with
 * the interrupt number and sent out to the PIC
 * to declare the interrupt finished */
#define EOI                 0x60


/* Mask all interrupts */
#define MASK_ALL 0xFF


uint32_t i8259_init(void);

uint32_t i8259_enable_irq(uint32_t num);

uint32_t i8259_disable_irq(uint32_t num);

uint32_t i8259_send_eoi(uint32_t num);

#endif