#include <types.h>
#include <x86_desc.h>
#include <lib.h>

#include "interrupts.h"
#include "i8259.h"

#define DEBUG 0

static irq_callback_t callback_head[16];

extern void (*irq_array[16])();

extern void sys_call();

uint32_t interrupts_init()
{
	printf("Initializing irq handlers...\n");
	int i;
	for (i = 32; i < 48; i++) {
		SET_IDT_ENTRY(idt[i], irq_array[i-32]);
		idt[i].seg_selector = KERNEL_CS;
		idt[i].reserved3 = 0;
		idt[i].reserved2 = 1;
		idt[i].reserved1 = 1;
		idt[i].size = 1;
		idt[i].reserved0 = 0;
		idt[i].dpl = 0;
		idt[i].present = 1;
	}

	SET_IDT_ENTRY(idt[0x80], sys_call);        
	idt[0x80].seg_selector = KERNEL_CS;    
	idt[0x80].reserved3 = 0;               
	idt[0x80].reserved2 = 1;               
	idt[0x80].reserved1 = 1;               
	idt[0x80].size = 1;                    
	idt[0x80].reserved0 = 0;               
	idt[0x80].dpl = 3;                     
	idt[0x80].present = 1;                 

	lidt(idt_desc_ptr);

	for (i = 0; i < 16; i++) {
		callback_head[i].handler = NULL;
		callback_head[i].next = NULL;
	}

	printf("Done!\n");
	return 0;
}

uint32_t attach_interrupt(int num, int dpl, irq_callback_t* callback)
{
	irq_callback_t* c = &callback_head[num];
	while (c->next != NULL)
		c = c->next;
	
	c->next = callback;

	return 0;
}

uint32_t handle_irq(uint32_t num)
{
	if (DEBUG)
		printf("Got an interrupt: %d\n", num);

	if (callback_head[num].next) {
		irq_callback_t* c = callback_head[num].next;
		while (c->handler != NULL) {
			c->handler();

			if (!(c = c->next))
				break;
		}

		return 0;
	}

	if (DEBUG)
		printf("No handler for irq %d\n", num);

	return 1;


}