#ifndef _EXECUTE_H
#define _EXECUTE_H

#include <types.h>

#include <Filesystem/filedescriptor.h>

#define EXEC_NOT_FOUND	-1
#define EXEC_NOT_ELF	-2

#define EXEC_BASE_ADDR	0x08000000
#define EXEC_LOAD_ADDR	0x08048000
#define EXEC_TMP_ADDR	0x0C000000

#define USER_ARGC (uint32_t*)0x83ffff8
#define USER_ARGV (uint8_t***)0x83ffffc

#define ET_NONE 0
#define ET_REL	1
#define ET_EXEC	2
#define ET_DYN	3
#define ET_CORE	4
#define ET_LOOS	0xfe00
#define ET_HIOS	0xfeff
#define ET_LOPROC 0xff00
#define ET_HIPROC 0xffff

#define EV_NONE 0
#define EV_CURRENT 1

#define EI_MAG0 0
#define EI_MAG1 1
#define EI_MAG2 2
#define EI_MAG3 3
#define EI_CLASS 4
#define EI_DATA 5
#define EI_VERSION 6
#define EI_OSABI 7
#define EI_ABIVERSION 8
#define EI_PAD 9
#define EI_NIDENT 16

#define ELFMAG0 0x7f
#define ELFMAG1 'E'
#define ELFMAG2 'L'
#define ELFMAG3 'F'

#define ELFCLASSNONE 0
#define ELFCLASS32 1
#define ELFCLASS64 2

#define ELFDATANONE 0
#define ELFDATA2LSB 1
#define ELFDATA2MSB 2

#define EI_NIDENT 16

typedef struct elf_header{
        uint8_t  e_ident[EI_NIDENT];
        uint16_t e_type;
        uint16_t e_machine;
        uint32_t e_version;
        uint32_t e_entry;
        uint32_t e_phoff;
        uint32_t e_shoff;
        uint32_t e_flags;
        uint16_t e_ehsize;
        uint16_t e_phentsize;
        uint16_t e_phnum;
        uint16_t e_shentsize;
        uint16_t e_shnum;
        uint16_t e_shstrndx;
} elf_header_t;

#define PT_NULL 0
#define PT_LOAD 1
#define PT_DYNAMIC 2
#define PT_INTERP 3
#define PT_NOTE 4
#define PT_SHLIB 5
#define PT_PHDR 6
#define PT_TLS 7
#define PT_LOOS 0x60000000
#define PT_HIOS 0x6fffffff
#define PT_LOPROC 0x70000000
#define PT_HIPROC 0x7fffffff

#define PF_X 1
#define PF_W 2
#define PF_R 4
#define PF_MASKOS 0x0ff00000
#define PF_MASKPROC 0xf0000000

typedef struct program_header {
	uint32_t p_type;
	uint32_t p_offset;
	uint32_t p_vaddr;
	uint32_t p_paddr;
	uint32_t p_filesz;
	uint32_t p_memsz;
	uint32_t p_flags;
	uint32_t p_align;
} program_header_t;

#define MAX_FD 8

typedef struct pcb {
	file_desc_t file_desc_array[8];
	int32_t is_vidmapped;
	uint32_t execute_return;
	int32_t pid;
	int32_t parent_pid;
	uint32_t parent_esp;
	uint32_t parent_ebp;
	uint32_t program_size;
	uint32_t program_break;
} pcb_t;

int32_t _sys_exec(uint8_t* name);
int32_t _sys_exit(uint32_t status);

pcb_t* get_pcb_ptr();

#endif