#include <types.h>
#include <lib.h>
#include <Filesystem/ext2.h>
#include <Filesystem/file.h>
#include <Filesystem/directory.h>

#include "syscalls.h"
#include "execute.h"

int32_t sys_read(int32_t fd, void* buf, int32_t len)
{
	pcb_t* pcb = get_pcb_ptr();
	pcb->file_desc_array[fd].file_ops->read(fd, buf, len);
	return 0;
}

int32_t sys_write(int32_t fd, const void* buf, int32_t len)
{
	pcb_t* pcb = get_pcb_ptr();
	pcb->file_desc_array[fd].file_ops->write(fd, buf, len);
	return 0;
}

int32_t sys_open(uint8_t* name)
{
	inode_t inode;
	printf("sys_open\n");
	uint32_t inode_num = ext2_parse_path_to_inode_num((int8_t*)name);
	if (!inode_num) {
		return -1;
	}

	int32_t fd = -1;

	ext2_get_inode(inode_num, &inode);
	printf("Inode %x type %x\n", inode_num, inode.type_permissions);

	if (inode.type_permissions & EXT2_DIR) {
		fd = directory_open(name);
	} else if (inode.type_permissions & EXT2_FILE) {
		fd = file_open(name);
	}

	return fd;
}

int32_t sys_close(int32_t fd)
{
	return -1;
}

int32_t sys_exec(uint8_t* name) 
{
	return _sys_exec(name);
}

int32_t sys_exit(uint32_t status) {
	return _sys_exit(status);
}