#define ASM 1

#include "multiboot.h"
#include "x86_desc.h"

.text
	/* Multiboot Header */
	.long MULTIBOOT_HEADER_MAGIC
	.long MULTIBOOT_HEADER_FLAGS
	.long -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	.rept 5
	.long 0
	.endr
	.long MULTIBOOT_VIDEO_MODE_TYPE
	.long MULTIBOOT_VIDEO_WIDTH
	.long MULTIBOOT_VIDEO_HEIGHT
	.long MULTIBOOT_VIDEO_DEPTH

/* Entrypopint to the kernel */
.globl start, _start

.align 4
start:
_start:
	/* make sure interrupts are off */
	cli
	jmp continue

continue:
	/* load the GDT */
	lgdt gdt_desc

	/* load CS with the new descriptor value */
	ljmp $KERNEL_CS, $keep_going

keep_going:
	/* set up ESP so we can have an initial stack */
	movl $0x800000, %esp

	/* set up the rest of the segment selector registers */
	movw $KERNEL_DS, %cx
	movw %cx, %ss
	movw %cx, %ds
	movw %cx, %es
	movw %cx, %fs
	movw %cx, %gs

	/* push the parameters that entry() expects (see kernel.c):
	 * eax = multiboot magic
	 * address of multiboot info struct */
	pushl %ebx
	pushl %eax

	/* jump to the c entrypoint to the kernel */
	call entry

halt:
	hlt
	jmp halt

