#include <types.h>
#include <lib.h>

#include "heap.h"
#include "paging.h"

uint32_t heap_create(int pid)
{
	map_large(HEAP_START, HEAP_PHYS - pid * _4MB);
	heap_header_t* header = (heap_header_t*)HEAP_START;
	header->size = 1;
	header->start = (heap_descriptor_t*)((uint32_t)header + sizeof(heap_header_t));
	header->start->size = _4MB - sizeof(heap_header_t) - sizeof(heap_descriptor_t);
	header->start->next = NULL;
	header->start->prev = NULL;
	header->start->flags &= ~HEAP_INUSE;
	return 0;
}


uint32_t heap_destroy()
{
	void* next = ((heap_header_t*)HEAP_START)->next_page;
	unmap_large(HEAP_START);
	
	// each process is guaranteed 1 large page, but other pages you need to figure out later
	// We'll try to free them anyway, if they exist.
	while (next) {
		void* page = ((heap_header_t*)next)->next_page;
		unmap_large((uint32_t)next);
		next = page;
	}
	return 0;
}

void* heap_alloc(uint32_t size)
{
	heap_descriptor_t* desc = ((heap_header_t*)HEAP_START)->start;
	while (desc->flags & 1) {
		desc = desc->next;
	}
	desc->flags |= HEAP_INUSE;
	desc->next = (heap_descriptor_t*)((uint32_t)desc + size + sizeof(heap_descriptor_t));
	return (void*)desc + sizeof(heap_descriptor_t);
}

uint32_t heap_free(void* addr)
{
	return -1;
}
