#include "paging.h"

void init_paging()
{
	//Initialize 4MB Page for Kernal Code
	page_directory[1].present = 1;
	page_directory[1].rw = 1;
	page_directory[1].ps = 1; //4mb page
	page_directory[1].g = 1; //dont flush kerneal code from virtual memory
	PDIR_SET_ADDR(KERNAL_PD_ENTRY, KERNAL_ADDR);

	/*	
	// it isn't guaranteed that the fb will always be at this address
	page_directory[0x3f4].present = 1;
	page_directory[0x3f4].rw = 1;
	page_directory[0x3f4].ps = 1;
	page_directory[0x3f4].g = 1;
	PDIR_SET_ADDR(0x3f4, 0xFD000000);
	*/
	

	//IA-32 Intel Page 57

	/* Page Size Extensions (bit 4 of CR4). Enables 4-MByte pages when set; restricts page
	* to 4 KBytes when clear. 
	*/
	asm volatile ("     \n\
	movl %%cr4, %%eax     \n\
	orl $0x10, %%eax     \n\
	movl %%eax, %%cr4"
	:
	:
	: "eax"
	);


	/* Moves Physical address of the base of the page directory Into CR3*/
	asm volatile ("     \n\
	movl %0, %%eax      \n\
	movl %%eax, %%cr3"
	:
	: "r" (page_directory)
	: "eax"
	);

	/*Enable PE and PG in CR0*/
	asm volatile ("         \n\
	movl %%cr0, %%eax         \n\
	orl $0x80000001, %%eax   \n\
	movl %%eax, %%cr0"
	:
	:
	: "eax"
	);

}

int32_t kmap_large(uint32_t v_addr, uint32_t p_addr)
{
	uint32_t dir_index = (uint32_t)v_addr >> DIR_BIT_OFF;

	if (dir_index > 1023)
		return -1;
	
	page_directory[dir_index].present = 1;
	page_directory[dir_index].rw = 1;
	page_directory[dir_index].us = 0;
	page_directory[dir_index].ps = 1;
	page_directory[dir_index].g = 1;
	PDIR_SET_ADDR(dir_index, p_addr);

	FLUSH_TLB();
	return 0;
}

int32_t map_large(uint32_t v_addr, uint32_t p_addr)
{
    uint32_t dir_index = (uint32_t)v_addr >> DIR_BIT_OFF;

    if (dir_index > 1023)
	return -1;

    /* fill page dir table accordingly */
    page_directory[dir_index].present = 1;
    page_directory[dir_index].rw = 1;
    page_directory[dir_index].us = 1;
    page_directory[dir_index].ps = 1; //4mb page
    PDIR_SET_ADDR(dir_index, p_addr);

    /* clear cache */
    FLUSH_TLB();
    

    return 0;
}

int32_t unmap_small(uint32_t v_addr);

int32_t unmap_large(uint32_t v_addr)
{
    uint32_t dir_index = (uint32_t)v_addr >> DIR_BIT_OFF;

    /* label as removed from page directory */
    page_directory[dir_index].present = 0;

    /* clear cache */
    FLUSH_TLB();

    return 0;
}

